@@ -1,8 +1,7 @@
 from concurrent.futures import CancelledError
 from logging import CRITICAL
-from pathlib import Path
 from threading import Event
-from typing import TYPE_CHECKING, Any, List, Optional
+from typing import TYPE_CHECKING, Any, Optional
 
 from robotcode.core.ignore_spec import DEFAULT_SPEC_RULES, GIT_IGNORE_FILE, ROBOT_IGNORE_FILE, IgnoreSpec, iter_files
 from robotcode.core.language import language_id
@@ -60,12 +59,16 @@ class RobotWorkspaceProtocolPart(RobotLanguageServerProtocolPart):
     def load_workspace_documents(self, sender: Any) -> None:
         with self._logger.measure_time(lambda: "loading workspace documents", context_name="load_workspace_documents"):
             try:
-                result: List[Path] = []
-
                 for folder in self.parent.workspace.workspace_folders:
                     config = self.parent.workspace.get_configuration(RobotCodeConfig, folder.uri)
 
                     extensions = [ROBOT_FILE_EXTENSION, RESOURCE_FILE_EXTENSION]
+
+                    exclude_patterns = [
+                        *self.parent.analysis_config.exclude_patterns,
+                        *(config.workspace.exclude_patterns or []),
+                    ]
+
                     with self.parent.window.progress("Collect sources", cancellable=False):
                         files = list(
                             filter(
@@ -75,7 +78,7 @@ class RobotWorkspaceProtocolPart(RobotLanguageServerProtocolPart):
                                     ignore_files=[ROBOT_IGNORE_FILE, GIT_IGNORE_FILE],
                                     include_hidden=False,
                                     parent_spec=IgnoreSpec.from_list(
-                                        [*DEFAULT_SPEC_RULES, *(config.workspace.exclude_patterns or [])],
+                                        [*DEFAULT_SPEC_RULES, *exclude_patterns],
                                         folder.uri.to_path(),
                                     ),
                                     verbose_callback=self._logger.debug,
@@ -84,9 +87,10 @@ class RobotWorkspaceProtocolPart(RobotLanguageServerProtocolPart):
                             )
                         )
 
-                    result.extend(files)
-
                     canceled = False
+                    self._logger.debug(
+                        lambda: f"Loading {len(files)} workspace documents", context_name="load_workspace_documents"
+                    )
                     with self.parent.window.progress(
                         "Load workspace", current=0, max=len(files), start=False, cancellable=False
                     ) as progress:
